@@ -13,8 +13,7 @@ try:
 except ImportError:
     pass
 
-from scipy.ndimage.filters import gaussian_filter
-from scipy.ndimage.interpolation import map_coordinates
+from scipy.ndimage import gaussian_filter, map_coordinates
 
 from .batch import Batch
 from .decorators import action, apply_parallel, inbatch_parallel
