@@ -8,10 +8,8 @@ import functools
 import logging
 import inspect
 
-try:
-    from numba import jit
-except ImportError:
-    jit = None
+from .utils_import import make_delayed_import
+jit = make_delayed_import('numba', attribute='jit')
 
 from .named_expr import P
 
@@ -46,7 +44,7 @@ def make_function(method, is_global=False):
     source = '\n'.join(source[start:])
 
     globs = globals() if is_global else method.__globals__.copy()
-    exec(source, globs)    # pylint:disable=exec-used
+    exec(source, globs)
 
     # Method with the same name might exist in various classes or modules
     # so a global function should have a unique name
@@ -65,7 +63,7 @@ def _workers_count():
     return cpu_count * 4
 
 
-def _make_action_wrapper_with_args(use_lock=None, no_eval=None):    # pylint: disable=redefined-outer-name
+def _make_action_wrapper_with_args(use_lock=None, no_eval=None):
     return functools.partial(_make_action_wrapper, use_lock=use_lock, no_eval=no_eval)
 
 def _make_action_wrapper(action_method, use_lock=None, no_eval=None):
@@ -215,7 +213,7 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
         if target in {'mpc', 'm'} and use_self:
             try:
                 mpc_method = make_function(method, is_global=True)
-            except Exception:  # pylint:disable=broad-except
+            except Exception: # noqa: BLE001, blind-except
                 mpc_method = None
 
         def _check_functions(self):
@@ -259,7 +257,7 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
                         result = future.result()
                     else:
                         result = future
-                except Exception as exce:  # pylint: disable=broad-except
+                except Exception as exce: # noqa: BLE001, blind-except
                     result = exce
                 finally:
                     all_results += [result]
@@ -274,7 +272,7 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
             return post_fn(all_results, *args, **kwargs)
 
         def _prepare_args(self, args, kwargs):
-            params = list()
+            params = []
 
             def _get_value(value, pos=None, name=None):
                 if isinstance(value, P):
@@ -301,11 +299,11 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
                isinstance(init_args[0], tuple) and isinstance(init_args[1], dict):
                 margs, mkwargs = init_args
             elif isinstance(init_args, dict):
-                margs = list()
+                margs = []
                 mkwargs = init_args
             else:
                 margs = init_args
-                mkwargs = dict()
+                mkwargs = {}
 
             margs = margs if isinstance(margs, (list, tuple)) else [margs]
 
@@ -395,7 +393,7 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
                 margs, mkwargs = _make_args(self, iteration, arg, args, kwargs, params)
                 futures.append(loop.create_task(method(*margs, **mkwargs)))
 
-            loop.run_until_complete(asyncio.gather(*futures, loop=loop, return_exceptions=True))
+            loop.run_until_complete(asyncio.gather(*futures, return_exceptions=True))
 
             return _call_post_fn(self, post_fn, futures, args, full_kwargs)
 
@@ -415,7 +413,7 @@ def inbatch_parallel(init, post=None, target='threads', _use_self=None, debug=Fa
                 else:
                     try:
                         one_ft = method(*margs, **mkwargs)
-                    except Exception as e:   # pylint: disable=broad-except
+                    except Exception as e: # noqa: BLE001, blind-except
                         one_ft = e
                 futures.append(one_ft)
 
@@ -460,7 +458,7 @@ def parallel(*args, use_self=None, **kwargs):
     return inbatch_parallel(*args, _use_self=use_self, **kwargs)
 
 
-def njit(nogil=True, parallel=True):  # pylint: disable=redefined-outer-name
+def njit(nogil=True, parallel=True):
     """ Fake njit decorator to use when numba is not installed """
     _, _ = nogil, parallel
     def njit_fake_decorator(method):
@@ -484,16 +482,22 @@ def mjit(*args, nopython=True, nogil=True, **kwargs):
     """
     def _jit(method):
         if jit is not None:
-            func = make_function(method)
-            func = jit(*args, nopython=nopython, nogil=nogil, **kwargs)(func)
+            try:
+                func = make_function(method)
+                func = jit(*args, nopython=nopython, nogil=nogil, **kwargs)(func)
+            except Exception: # noqa: BLE001, blind-except
+                # the source is not available or not compilable
+                func = method
+                logging.warning('The method cannot be compiled because the source code is not available. '
+                                'This causes a severe performance degradation for method %s',
+                                method.__name__)
         else:
             func = method
             logging.warning('numba is not installed. This causes a severe performance degradation for method %s',
                             method.__name__)
 
         @functools.wraps(method)
-        def _wrapped_method(self, *args, **kwargs):
-            _ = self
+        def _wrapped_method(_, *args, **kwargs):
             return func(None, *args, **kwargs)
         return _wrapped_method
 
