@@ -1,5 +1,4 @@
 """ Contains pipeline class """
-# pylint:disable=undefined-variable
 import sys
 import time
 from functools import partial
@@ -7,6 +6,7 @@ import threading
 import asyncio
 import logging
 import warnings
+from time import perf_counter
 
 from .base import Baseset
 from .config import Config
@@ -20,7 +20,7 @@ from .variables import VariableDirectory
 from .models.metrics import (ClassificationMetrics, SegmentationMetricsByPixels,
                              SegmentationMetricsByInstances, RegressionMetrics, Loss)
 
-from ._const import *       # pylint:disable=wildcard-import
+from ._const import *
 from .utils import save_data_to
 from .utils_random import make_rng
 from .pipeline_executor import PipelineExecutor
@@ -54,8 +54,6 @@ def hashable(x):
 class Pipeline:
     """ Pipeline """
     def __init__(self, dataset=None, config=None, pipeline=None, actions=None, strict=False, proba=None, repeat=None):
-        # pylint: disable=protected-access
-
         if pipeline is None:
             self.dataset = dataset
             self.config = config or {}
@@ -72,7 +70,7 @@ class Pipeline:
             config = config or {}
             _config = pipeline.config or {}
             self.config = {**config, **_config}
-            self._actions = actions or pipeline._actions[:]
+            self._actions = actions or pipeline._actions[:]  # noqa: SLF001; private-member-access
             if self.num_actions == 1:
                 if proba is not None:
                     if self.get_last_action_repeat() is None:
@@ -80,11 +78,11 @@ class Pipeline:
                 elif repeat is not None:
                     if self.get_last_action_proba() is None:
                         self._actions[-1]['repeat'] = mult_option(repeat, self.get_last_action_repeat())
-            self._lazy_run = pipeline._lazy_run
+            self._lazy_run = pipeline._lazy_run  # noqa: SLF001; private-member-access
             self.variables = pipeline.variables.copy()
             self.strict = pipeline.strict
             self.models = pipeline.models.copy()
-            self._namespaces = pipeline._namespaces
+            self._namespaces = pipeline._namespaces  # noqa: SLF001; private-member-access
             self.before = pipeline.before.copy()
             self.before.pipeline = self
             self.after = pipeline.after.copy()
@@ -102,6 +100,7 @@ class Pipeline:
         self.random_seed = None
 
         self._profiler = None
+        self.run_time = None
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -109,6 +108,7 @@ class Pipeline:
         state.pop('_local')
         state['_profiler'] = None
         state['_batch_generator'] = None
+        state['iter_params'] = None
         return state
 
     def __setstate__(self, state):
@@ -163,15 +163,14 @@ class Pipeline:
     @classmethod
     def concat(cls, pipe1, pipe2):
         """ Create a new pipeline concatenating two given pipelines """
-        # pylint: disable=protected-access
         new_p1 = cls.from_pipeline(pipe1)
-        new_p1._actions += pipe2._actions[:]
+        new_p1._actions += pipe2._actions[:]  # noqa: SLF001; private-member-access
         new_p1.config.update(pipe2.config)
         new_p1.variables += pipe2.variables
         new_p1.models += pipe2.models
         if new_p1.dataset is None:
             new_p1.dataset = pipe2.dataset
-        new_p1._lazy_run = new_p1._lazy_run or pipe2._lazy_run
+        new_p1._lazy_run = new_p1._lazy_run or pipe2._lazy_run  # noqa: SLF001; private-member-access
         new_p1.before = pipe1.before.concat(pipe1.before, pipe2.before)
         new_p1.before.pipeline = new_p1
         new_p1.after = pipe1.after.concat(pipe1.after, pipe2.after)
@@ -331,13 +330,13 @@ class Pipeline:
         self._actions.append({'name': PIPELINE_ID, 'pipeline': pipeline, 'proba': proba, 'repeat': repeat})
 
 
-    LINE_LENGTH = 80
-    ITEM_LENGTH = 40
-    PREFIX = '\033[1m\033[4m'
 
-    def __str__(self):
+    def repr(self, line_length=80, item_length=40, supress_color=False):
         """ Textual representation of a pipeline: list all actions and their keyword parameters. """
         # TODO: with the help of `inspect` and `get_method` method, can match the `args` to their actual names
+        prefix = '' if supress_color else '\033[1m\033[4m'
+        postfix = '' if supress_color else '\033[0m'
+
         msg = []
         for i, action in enumerate(self._actions):
             name = self.get_action_name(action)
@@ -354,22 +353,24 @@ class Pipeline:
             for key, value in kwargs.items():
                 # Control the length of one parameter value
                 value_ = repr(value)
-                value_ = value_ if len(value_) < self.ITEM_LENGTH else '[...]'
+                value_ = value_ if len(value_) < item_length else '[...]'
                 str_kwargs += f'{key}={value_}, '
 
                 # Control the length of current line
-                if len(str_kwargs) + len(indent) > self.LINE_LENGTH:
+                if len(str_kwargs) + len(indent) > line_length:
                     lines_kwargs.append(str_kwargs[:-2])
                     str_kwargs = ''
             if str_kwargs:
                 lines_kwargs.append(str_kwargs[:-2])
             str_kwargs = f'\n{indent}'.join(lines_kwargs)
 
-            action_msg = f'#{i} {self.PREFIX}{name}\033[0m({str_kwargs})'
+            action_msg = f'#{i} {prefix}{name}{postfix}({str_kwargs})'
             msg.append(action_msg)
 
         return '\n'.join(msg)
 
+    def __str__(self):
+        return self.repr()
 
     @property
     def index(self):
@@ -865,7 +866,7 @@ class Pipeline:
         if self._needs_exec(batch, action):
             repeat = self._eval_expr(action['repeat'], batch=batch) or 1
             for _ in range(repeat):
-                batch = self._exec_all_actions(batch, action['pipeline']._actions)  # pylint: disable=protected-access
+                batch = self._exec_all_actions(batch, action['pipeline']._actions)  # noqa: SLF001; private-member-access
         return batch
 
     def _exec_all_actions(self, batch, actions=None, iteration=None):
@@ -876,6 +877,7 @@ class Pipeline:
         batch.iteration = iteration
 
         for action in actions:
+            action_time = 0
             if self._profiler:
                 self._profiler.enable()
 
@@ -883,11 +885,13 @@ class Pipeline:
                 pass
             elif action['name'] in [JOIN_ID, MERGE_ID]:
                 join_batches = []
-                for pipe in action['pipelines']:   # pylint: disable=not-an-iterable
+                for pipe in action['pipelines']:
                     if action['mode'] == 'i':
                         jbatch = pipe.create_batch(batch.index)
                     elif action['mode'] == 'n':
                         jbatch = pipe.next_batch()
+                    else:
+                        raise ValueError(f"Wrong mode value: {action['mode']}")
                     join_batches.append(jbatch)
                 join_batches = tuple(join_batches)
 
@@ -912,7 +916,8 @@ class Pipeline:
 
             if self._profiler:
                 name = self.get_action_name(action, add_index=True)
-                self._profiler.disable(batch.iteration, name, batch_id=id(batch), action_time=action_time)
+                self._profiler.disable(iteration=batch.iteration, name=name,
+                                       batch_id=id(batch), action_time=action_time)
 
         return batch
 
@@ -1381,9 +1386,8 @@ class Pipeline:
 
     def rebatch(self, batch_size, merge=None, components=None, batch_class=None):
         """ Set the output batch size """
-        # pylint:disable=protected-access
         new_p = type(self)(self.dataset)
-        return new_p._add_action(REBATCH_ID, _args=dict(batch_size=batch_size, pipeline=self, merge=merge,
+        return new_p._add_action(REBATCH_ID, _args=dict(batch_size=batch_size, pipeline=self, merge=merge,  # noqa: SLF001; private-member-access
                                                         components=components, batch_class=batch_class))
 
     def reset(self, *args, profile=False, seed=None):
@@ -1445,11 +1449,12 @@ class Pipeline:
 
         self.random_seed = seed
 
-        if profile == 2 or isinstance(profile, str) and 'detailed'.startswith(profile):
-            self._profiler = PipelineProfiler(detailed=True)
-        elif profile == 1 or profile is True:
-            self._profiler = PipelineProfiler(detailed=False)
-        else: # 0, False, None
+        if self._profiler is None:
+            if profile == 2 or isinstance(profile, str) and 'detailed'.startswith(profile):
+                self._profiler = PipelineProfiler(detailed=True)
+            elif profile in [1, True]:
+                self._profiler = PipelineProfiler(detailed=False)
+        if profile in [0, False, None]:
             self._profiler = None
 
 
@@ -1636,6 +1641,7 @@ class Pipeline:
         --------
         :meth:`~.Pipeline.gen_batch`
         """
+        start_time = perf_counter()
         if kwargs.pop('lazy', False):
             # if lazy is passed, then store params for later execution
             self._lazy_run = args, kwargs
@@ -1650,7 +1656,9 @@ class Pipeline:
             warnings.warn('Batch generation will never stop as ' \
                           'n_epochs=None and n_iters=None', RuntimeWarning)
 
-        return PipelineExecutor(self).run(*args_value, **kwargs_value)
+        PipelineExecutor(self).run(*args_value, **kwargs_value)
+        self.run_time = perf_counter() - start_time
+        return self
 
     def run_now(self, *args, **kwargs):
         """ Execute pipeline immediately """
