@@ -1,9 +1,11 @@
 """ Plot primitives. """
 from copy import copy
 from datetime import datetime
+from functools import wraps
 from itertools import cycle
 from numbers import Number
 from warnings import warn
+from multiprocessing import Process
 
 import numpy as np
 
@@ -23,6 +25,29 @@ from ..utils import to_list
 
 
 
+# Decorators
+def detachable(func):
+    """ Run `func` in a daemon process without result return.
+
+    Note, the decorator intercept the `detach` argument from the `func`.
+    """
+    @wraps(func)
+    def _wrapper(*args, **kwargs):
+        detach = kwargs.get('detach', False)
+
+        if detach is True:
+            process = Process(target=func, args=args, kwargs=kwargs,
+                              daemon=True, name=f'daemon_for_{func.__qualname__}')
+            process.start()
+            return None
+
+        result = func(*args, **kwargs)
+        return result
+
+    return _wrapper
+
+
+
 class Layer:
     """ Implements subplot layer, storing plotted object and redirecting call to corresponding matplotlib methods.
         Also handles data transformations in `image` mode, re-applying it to given data on `set_data` call.
@@ -78,8 +103,7 @@ class Layer:
 
     def dilate(self, data):
         """ Apply dilation to array. """
-        # pylint: disable=import-outside-toplevel
-        import cv2
+        from .morphology import dilate
         dilation_config = self.config.get('dilate', False)
 
         default_kernel = np.ones((3, 1), dtype=np.uint8)
@@ -92,7 +116,7 @@ class Layer:
                 dilation_config = {'kernel': np.ones(dilation_config, dtype=np.uint8)}
             elif 'kernel' in dilation_config and isinstance(dilation_config['kernel'], tuple):
                 dilation_config['kernel'] = np.ones(dilation_config['kernel'], dtype=np.uint8)
-            data = cv2.dilate(data.astype(np.float32), **dilation_config)
+            data = dilate(data.astype(np.float32), **dilation_config)
         return data
 
     def mask(self, data):
@@ -108,6 +132,8 @@ class Layer:
                     condition_mask = evaluate_str_comparison(data, condition)
                 elif callable(condition):
                     condition_mask = condition(data)
+                else:
+                    raise ValueError(f'Wrong condition type: {type(condition)}')
                 mask = np.logical_or(mask, condition_mask)
             data = np.ma.array(data, mask=mask)
         return data
@@ -116,6 +142,7 @@ class Layer:
         """ Calculate running average on given data with provided window. """
         window = self.config.get('window')
         if window is not None and window < len(data):
+            data = np.array(data, dtype='float32')
             data = convolve(data, np.ones(window), mode='nearest') / window
             return data
         return None
@@ -169,8 +196,7 @@ class Layer:
             data = self.preprocess(data)
             self.main_object.set_data(data)
 
-            vmin = self.config.get('vmin', np.nanmin(data))
-            vmax = self.config.get('vmax', np.nanmax(data))
+            vmin, vmax = self._parse_vrange(data)
             self.main_object.set_clim([vmin, vmax])
 
         if self.mode == 'histogram':
@@ -184,6 +210,31 @@ class Layer:
         if self.mode == 'loss':
             raise NotImplementedError("Updating layer data is not in supported in 'loss' mode. ")
 
+    def _parse_vrange(self, data):
+        """ Parse vmin and vmax values from the `self.config`. """
+        vmin = self.config.get('vmin', np.nanmin(data))
+        vmax = self.config.get('vmax', np.nanmax(data))
+        vrange = self.config.get('vrange', None)
+
+        # Parse vmin, vmax
+        if isinstance(vmin, str):
+            vmin = np.nanquantile(data, q=float(vmin))
+        if isinstance(vmax, str):
+            vmax = np.nanquantile(data, q=float(vmax))
+
+        # Parse vrange
+        if vrange is True:
+            vrange = max(abs(np.nanmin(data)), abs(np.nanmax(data)))
+        elif isinstance(vrange, str):
+            vrange = abs(np.nanquantile(data, q=(float(vrange), 1-float(vrange)))).max()
+
+        if vrange is not None:
+            if isinstance(vrange, (list, tuple, np.ndarray)):
+                vmin, vmax = vrange
+            else:
+                vmin, vmax = -vrange, vrange
+        return vmin, vmax
+
     # Plotting methods
     def image(self, data):
         """ Display data as an image. """
@@ -205,6 +256,10 @@ class Layer:
         image_keys = ['alpha', 'vmin', 'vmax', 'extent']
         image_config = self.config.filter(keys=image_keys, prefix='image_')
 
+        vmin, vmax = self._parse_vrange(data)
+        image_config['vmin'] = vmin
+        image_config['vmax'] = vmax
+
         image = self.ax.imshow(data, cmap=cmap, **image_config)
 
         return [image]
@@ -214,6 +269,10 @@ class Layer:
         matrix_keys = ['cmap', 'vmin', 'vmax']
         matrix_config = self.config.filter(keys=matrix_keys, prefix='matrix_')
 
+        vmin, vmax = self._parse_vrange(data)
+        matrix_config['vmin'] = vmin
+        matrix_config['vmax'] = vmax
+
         matrix = self.ax.matshow(data, **matrix_config)
 
         label_keys = ['size', 'color', 'bbox', 'format']
@@ -236,6 +295,8 @@ class Layer:
                     for dtype, dtype_formatter in label_format.items():
                         if isinstance(value, dtype):
                             formatter = dtype_formatter
+                else:
+                    raise ValueError(f'Wrong label_format: {type(label_format)}')
 
                 text = format(value, formatter)
 
@@ -484,7 +545,6 @@ class Subplot:
     # Annotation methods
     def annotate(self, mode):
         """ Apply requested annotation functions to subplot with parameters from subplot config. """
-        # pylint: disable=too-many-branches
         annotations = {}
 
         if not self.ax.axison:
@@ -560,6 +620,10 @@ class Subplot:
         if tick_config:
             self.ax.tick_params(**tick_config)
 
+        # Change visibility of xaxis/yaxis
+        self.ax.get_xaxis().set_visible(self.config.get('xaxis_visible', True))
+        self.ax.get_yaxis().set_visible(self.config.get('yaxis_visible', True))
+
         # Change scale of axis, if needed
         if self.config.get('log') or self.config.get('log_loss'):
             self.ax.set_yscale('log')
@@ -619,6 +683,9 @@ class Subplot:
         grid = self.config.get('grid', None)
         grid_config = self.config.filter(prefix='grid_')
 
+        if grid in (None, False):
+            self.ax.grid(False)
+
         if grid in ('minor', 'both'):
             minor_grid_config = self.config.filter(prefix='minor_grid_')
             minor_grid_config = minor_grid_config.update(grid_config, skip_duplicates=True)
@@ -653,7 +720,8 @@ class Subplot:
 
         return colorbar
 
-    def add_legend(self, mode='image', label=None, color='none', alpha=1, size=15, **kwargs):
+    def add_legend(self, mode='image', label=None, color='none', alpha=1,
+                   size=15, family='sans-serif', properties=None, **kwargs):
         """ Add patches to subplot legend.
 
         Parameters
@@ -671,10 +739,18 @@ class Subplot:
         alpha : number or list of numbers from 0 to 1
             Legend handles opacity.
         size : int
-            Legend size.
+            Legend text font size.
+        family : str
+            Legent text font family.
+        properties : dict
+            Legend font parameters, must be valid for `matplotlib.font_manager.FontProperties`.
         kwargs : misc
             For `matplotlib.legend`.
         """
+        if properties is None:
+            properties = {}
+        properties = {'size': size, 'family': family, **properties}
+
         # get legend that already exists
         legend = self.ax.get_legend()
         old_handles = getattr(legend, 'legendHandles', [])
@@ -689,6 +765,7 @@ class Subplot:
         for label_item, label_color, label_alpha in zip(labels, colors, alphas):
             if label_item is None:
                 continue
+
             if isinstance(label_item, str):
                 if mode in ('image', 'histogram'):
                     if is_color_like(label_color):
@@ -698,6 +775,8 @@ class Subplot:
                         handler_map[PatchCollection] = ColorMappingHandler()
                 elif mode in ('curve', 'loss'):
                     handle = Line2D(xdata=[0], ydata=[0], color=label_color, alpha=label_alpha, label=label_item)
+                else:
+                    raise ValueError(f"Mode must be 'image', 'histogram', 'curve' or 'loss' but mode is {mode}")
                 new_handles.append(handle)
             elif not label_item.get_label().startswith('_'):
                 new_handles.append(label_item)
@@ -705,7 +784,7 @@ class Subplot:
         if len(new_handles) > 0:
             # extend existing handles and labels with new ones
             handles = old_handles + new_handles
-            legend = self.ax.legend(prop={'size': size}, handles=handles,  handler_map=handler_map, **kwargs)
+            legend = self.ax.legend(prop=properties, handles=handles,  handler_map=handler_map, **kwargs)
 
         return legend
 
@@ -742,6 +821,8 @@ class Subplot:
             locator = AutoMinorLocator
         elif grid_type == 'major':
             locator = MaxNLocator
+        else:
+            raise ValueError(f'Unknown grid_type: {grid_type}')
 
         if isinstance(frequency, tuple):
             x_frequency, y_frequency = frequency
@@ -774,10 +855,10 @@ class Subplot:
 
         self.annotations = {}
 
-        self.ax.clear()
         for layer in self.layers:
             for obj in layer.objects:
                 obj.remove()
+        self.ax.clear()
         self.layers = []
 
 
@@ -870,6 +951,11 @@ class Plot:
     fix_config : bool
         If False, every time `plot` is called update config with provided keyword arguments, replacing older parameters.
         If True, fix plotter config as provided on initialization. Usefull, if one want to reuse this config on updates.
+    detach : {True, False, 'save'}, default: False
+        Whether to use run `plot` in a daemon process.
+        If False, then don't use any daemon processes.
+        If True, then run :meth:`~.plot` in a daemon process.
+        If 'save', then run :meth:`~.save` (called from the :meth:`~.plot`) in a daemon process.
     kwargs :
         - For one of `image`, `histogram`, `curve`, `loss` methods of `Layer` (depending on chosen mode).
             Parameters and data nestedness levels must match if they are lists meant for differents subplots/layers.
@@ -916,13 +1002,24 @@ class Plot:
         If a callable, must return boolean mask with the same shape as original image that mark image pixels to mask.
         If a tuple, contain any combination of items of types above.
     cmap : str or matplotlib colormap object
-        Сolormap to display single-channel images with. Must be valid matplotlib colormap (e.g. 'ocean', 'tab20b').
+        Colormap to display single-channel images with. Must be valid matplotlib colormap (e.g. 'ocean', 'tab20b').
     mask_color : string or tuple of 3 or 4 numbers
         Color to display masked values with. Must be valid matplotlib color (e.g. 'salmon', '#120FA3', (0.3, 0.4, 0.5)).
     alpha : number in (0, 1) range
         Image opacity (0 means fully transparent, i.e. invisible, 1 - totally opaque). Useful when `combine='overlay'`.
-    vmin, vmax : number
+    vmin, vmax : number, str, None
         Limits for normalizing image into (0, 1) range. Values beyond range are clipped (default matplotlib behaviour).
+        Note, if you provide a str it must be a repr of a float value in (0, 1) range.
+        If number, then it is used as a clipping value.
+        If str, then `float(str)` data quantile will be clipping value.
+        If None, then vmin and vmax are evaluated from data.
+    vrange : number, str, True, sequence of two numbers optional
+        Way to evaluate symmetric zero-centered vmin and vmax values.
+        Note, if you provide a str it must be a repr of a float value in (0, 1) range.
+        If sequence of two numbers, then this values are used as low and high clipping values.
+        If number, then (-vrange, vrange) are used as clipping values.
+        If str, then vrange is max abs value from (float(str), 1-float(str)) quantiles.
+        If True, then vrange is max abs value from data extremes.
     extent : tuple of 4 numbers
         The bounding box in data coordinates that the image will fill.
     image_{parameter} : misc
@@ -1283,6 +1380,9 @@ class Plot:
         elif mode in ('curve', 'loss'):
             ratio = 1 / 3 / ncols * nrows
 
+        else:
+            raise ValueError(f"Mode must be 'image', 'histogram', 'curve' or 'loss' but mode is {mode}")
+
         return ratio
 
     @classmethod
@@ -1472,8 +1572,9 @@ class Plot:
         return defaults
 
     # Plotting delegator
+    @detachable
     def plot(self, data=None, combine='overlay', mode='image', show=True, force_show=False, save=False,
-             axes=None, positions=None, n_subplots=None, adjust_figsize='image', **kwargs):
+             axes=None, positions=None, n_subplots=None, adjust_figsize='image', detach=False, **kwargs):
         """ Plot data on subplots.
 
         If a first call (from `__init__`), parse axes from kwargs if they are provided, else create them.
@@ -1496,7 +1597,7 @@ class Plot:
                 self.config = PlotConfig()
             outer_config = PlotConfig(kwargs)
             if axes is not None:
-                msg = "Subplots already created and new axes cannot bespecified."
+                msg = "Subplots already created and new axes cannot be specified."
                 raise ValueError(msg)
 
         positions = list(range(len(data))) if positions is None else to_list(positions)
@@ -1530,7 +1631,8 @@ class Plot:
             self.close()
 
         if save or 'savepath' in self.config:
-            self.save()
+            detach_save = detach=='save'
+            self.save(detach=detach_save)
 
         return self
 
@@ -1585,8 +1687,15 @@ class Plot:
 
         default_savepath = datetime.now().strftime('%Y-%m-%d_%H:%M:%S.png')
         savepath = save_config.pop('savepath', default_savepath)
+        detach = save_config.pop('detach', False)
 
-        self.figure.savefig(fname=savepath, **save_config)
+        if savepath:
+            if detach:
+                process = Process(target=self.figure.savefig, kwargs={'fname': savepath, **save_config},
+                                  daemon=True, name=f'daemon_for_{self.save.__qualname__}')
+                process.start()
+            else:
+                self.figure.savefig(fname=savepath, **save_config)
 
     def close(self):
         """ Close figure. """
