@@ -59,7 +59,7 @@ class ConfigAlias:
     """
     def __init__(self, config=None):
         if isinstance(config, ConfigAlias):
-            _config = config._config
+            _config = config._config  # noqa: SLF001; private-member-access
         else:
             _config = []
             if isinstance(config, (dict, Config)):
@@ -256,7 +256,7 @@ class Domain:
         self.create_id_prefix = False
         self.random_state = None
 
-        self.values_indices = dict()
+        self.values_indices = {}
 
     def _get_all_options_names(self):
         options = []
@@ -325,7 +325,7 @@ class Domain:
         """ Set domain update parameters. """
         if isinstance(when, (int, str)):
             when = [when]
-        iter_kwargs = dict()
+        iter_kwargs = {}
         for attr in ['n_items', 'n_reps', 'repeat_each']:
             iter_kwargs[attr] = kwargs.pop(attr) if attr in kwargs else getattr(self, attr)
         self.updates.append({
@@ -364,7 +364,7 @@ class Domain:
             lengthes = [len(values) for _, values in cube if isinstance(values, (list, tuple, np.ndarray))]
             if len(lengthes) == 0:
                 return None
-            size += np.product(lengthes)
+            size += np.prod(lengthes)
         return size
 
     def __len__(self):
@@ -485,7 +485,7 @@ class Domain:
                 else:
                     additional = ConfigAlias()
                 while self.n_items is None or i < self.n_items:
-                    res = next(iterator) + additional # pylint: disable=stop-iteration-return
+                    res = next(iterator) + additional
                     if self.create_id_prefix:
                         res.set_prefix(keys, n_digits=int(self.create_id_prefix))
                     yield res
@@ -592,7 +592,7 @@ class Domain:
         for value in values:
             if self.create_id_prefix:
                 n_digits = self.create_id_prefix if self.create_id_prefix is not True else 1
-                option_values = self.values_indices.get(name.alias, dict())
+                option_values = self.values_indices.get(name.alias, {})
                 current_index = option_values.get(value.alias, len(option_values))
                 option_values[value.alias] = current_index
                 self.values_indices[name.alias] = option_values
