@@ -1,4 +1,3 @@
-#pylint:disable=logging-fstring-interpolation
 """ Experiment and corresponding classes. """
 
 import os
@@ -349,14 +348,14 @@ class Experiment:
         self._is_failed = False # was an exception raised or not
 
         self.last = False
-        self.outputs = dict()
+        self.outputs = {}
         self.storage = None
         self.has_dump = False # does unit has any dump actions or not
         self.name = None # name of the executor/research
         self.dump_results = None
         self.loglevel = None
         self.monitor = None
-        self.id = None #pylint:disable=invalid-name
+        self.id = None
         self.index = None
         self.config_alias = None
         self.config = None
@@ -584,7 +583,7 @@ class Experiment:
             return None
         return _explicit_call(method, name, self)
 
-    def save(self, src, dst, when=1, save_output_dict=False, copy=False): #pylint:disable=redefined-outer-name
+    def save(self, src, dst, when=1, save_output_dict=False, copy=False):
         """ Save something to research results.
 
         Parameters
@@ -733,6 +732,7 @@ class Experiment:
             'stderr', self.storage.stderr_file, self.executor.storage.stderr_file
         )
 
+        unit_time = 0
         with context_manager_out, context_manager_err:
             if self.is_alive or name.startswith('__'):
                 if self.profiler:
@@ -744,7 +744,7 @@ class Experiment:
                 exception = (StopIteration, KeyboardInterrupt) if self.debug else Exception
                 try:
                     self.outputs[name], unit_time = self.actions[name](iteration, n_iters, last=self.last)
-                except exception as e: #pylint:disable=broad-except
+                except exception as e:
                     self.is_failed = True
                     self.last = True
                     if isinstance(e, StopIteration):
@@ -835,7 +835,7 @@ class Executor:
             self.n_branches = len(configs)
 
         self.configs = configs or [Config() for _ in range(self.n_branches)]
-        self.executor_config = Config(executor_config or dict())
+        self.executor_config = Config(executor_config or {})
         self.branches_configs = branches_configs or [Config() for _ in range(self.n_branches)]
         self.branches_configs = [Config(config) for config in self.branches_configs]
         self.n_iters = n_iters
@@ -879,11 +879,11 @@ class Executor:
             'dump_results': False,
             'finalize': False
         }
-        for attr in defaults:
+        for attr, value_ in defaults.items():
             if self.research:
                 value = getattr(self.research, attr)
             else:
-                value = kwargs.get(attr, defaults[attr])
+                value = kwargs.get(attr, value_)
             setattr(self, attr, value)
 
     def create_experiments(self):
@@ -914,8 +914,8 @@ class Executor:
                     if unit.root or len(self.experiments) == 1:
                         self.call_root(iteration, unit_name)
                     else:
-                        self.parallel_call(iteration, unit_name, target=self.target, debug=self.debug) #pylint:disable=unexpected-keyword-arg
-                if not any([experiment.is_alive for experiment in self.experiments]):
+                        self.parallel_call(iteration, unit_name, target=self.target, debug=self.debug)
+                if not any(experiment.is_alive for experiment in self.experiments):
                     break
                 if self.research:
                     for experiment in self.experiments:
@@ -985,7 +985,7 @@ def _create_instance(experiments, item_name):
     for e in experiments:
         e.instances[item_name] = instance
 
-def _get_input(x, copy, *args, **kwargs): #pylint:disable=redefined-outer-name
+def _get_input(x, copy, *args, **kwargs):
     _ = args, kwargs
     return deepcopy(x) if copy else x
 
