@@ -1,4 +1,3 @@
-#pylint: disable=super-init-not-called
 """ Research profilers. """
 
 import os
@@ -6,12 +5,10 @@ import glob
 import warnings
 import multiprocess as mp
 
-try:
-    import pandas as pd
-except ImportError:
-    from . import _fake as pd
-
 from ..profiler import Profiler
+from ..utils_import import make_delayed_import
+pd = make_delayed_import('pandas')
+
 
 class ExperimentProfiler(Profiler):
     """ Profiler for Research experiments. """
@@ -48,11 +45,11 @@ class ExperimentProfiler(Profiler):
 
         if per_iter is False and detailed is False:
             groupby = ['experiment', 'unit', 'iter'] if per_experiment else ['unit', 'iter']
-            columns = columns or ['total_time', 'eval_time', 'unit_time']
+            columns = columns or ['total_time', 'unit_time']
             sortby = sortby or ('total_time', 'sum')
             aggs = {key: ['sum', 'mean', 'max'] for key in columns}
-            result = (self.profile_info.groupby(groupby)[columns].mean()
-                      .groupby(groupby[:-1]).agg(aggs)
+            result = (self.profile_info.groupby(groupby)[columns].mean(numeric_only=True)
+                      .groupby(groupby[:-1]).agg(aggs, numeric_only=True)
                       .sort_values(sortby, ascending=False))
 
             if per_experiment:
@@ -64,7 +61,7 @@ class ExperimentProfiler(Profiler):
             sortby = sortby or ('tottime', 'sum')
             aggs = {key: ['sum', 'mean', 'max'] for key in columns}
             level = [0, 1] if per_experiment else 0
-            result = (self.profile_info.reset_index().groupby(groupby).agg(aggs)
+            result = (self.profile_info.reset_index().groupby(groupby).agg(aggs, numeric_only=True)
                     .sort_values([*groupby[:-1], sortby], ascending=[True] * (len(groupby)-1) + [False])
                     .groupby(level=level).apply(lambda df: df.iloc[:limit].reset_index(level)))
 
@@ -72,9 +69,9 @@ class ExperimentProfiler(Profiler):
             groupby = groupby or ['iter', 'unit']
             groupby = ['experiment', *groupby] if per_experiment else groupby
 
-            columns = columns or ['unit', 'total_time', 'eval_time', 'unit_time']
+            columns = columns or ['unit', 'total_time', 'unit_time']
             sortby = sortby or 'total_time'
-            result = (self.profile_info.reset_index().groupby(groupby)[columns].mean()
+            result = (self.profile_info.reset_index().groupby(groupby)[columns].mean(numeric_only=True)
                       .sort_values([*groupby[:-1], sortby], ascending=[True] * (len(groupby)-1) + [False]))
 
         elif per_iter is True and detailed is True:
@@ -92,8 +89,8 @@ class ExperimentProfiler(Profiler):
 class ExecutorProfiler(ExperimentProfiler):
     """ Profiler for Executor experiments. """
     def __init__(self, experiments):
-        self.profilers = [experiment._profiler for experiment in experiments]
-        self.detailed = experiments[0]._profiler.detailed
+        self.profilers = [experiment._profiler for experiment in experiments]  # noqa: SLF001; private-member-access
+        self.detailed = experiments[0]._profiler.detailed  # noqa: SLF001; private-member-access
 
     @property
     def profile_info(self):
