@@ -6,14 +6,17 @@ import io
 import contextlib
 import warnings
 from copy import deepcopy
+
 import dill
 import numpy as np
-import pandas as pd
 
 from ..plotter import plot
 from ..plotter.plot import Subplot
 from ..utils import to_list
 
+from ..utils_import import make_delayed_import
+pd = make_delayed_import('pandas')
+
 
 class MultiOut:
     """ Wrapper for several outputs streams. """
@@ -131,7 +134,7 @@ def parse_name(name):
 def generate_id(config, random, create_prefix=False):
     """ Generate id for experiment. """
     name = config.alias()['_prefix'] if create_prefix else ''
-    name += hashlib.md5(config.alias(as_string=True).encode('utf-8')).hexdigest()[:8]
+    name += hashlib.md5(config.alias(as_string=True).encode('utf-8')).hexdigest()[:8] # noqa: S324; hashlib-insecure-hash-function
     name += ''.join(str(i) for i in random.integers(10, size=8))
     return name
 
@@ -142,11 +145,11 @@ def create_output_stream(redirect, dump=False, filename=None, path=None, common=
         if redirect in values:
             if dump:
                 filename = os.path.join(path, filename)
-                file = open(filename, 'a')
+                file = open(filename, 'a', encoding='utf-8')
             else:
                 file = io.StringIO()
         else:
-            file = open(os.devnull, 'w')
+            file = open(os.devnull, 'w', encoding='utf-8')
     else:
         file = contextlib.nullcontext()
     return file
@@ -316,6 +319,8 @@ def plot_research(df, variables=None, subplots=None, aggregate=None, aggregate_f
             ncols, nrows = 1, len(index)
         elif layout == 'horizontal':
             ncols, nrows = len(index), 1
+        else:
+            raise ValueError(f'Unknown layout: {layout}')
 
         plot_config = {**plot_config, 'ncols': ncols, 'nrows': nrows, 'ratio': nrows / ncols}
 
