@@ -1,4 +1,3 @@
-# pylint: disable=too-few-public-methods, method-hidden
 """ Contains Sampler-classes. """
 
 import warnings
@@ -76,7 +75,7 @@ def arithmetize(cls):
     return cls
 
 @arithmetize
-class Sampler():
+class Sampler:
     """ Base class Sampler that implements algebra of Samplers.
 
     Attributes
@@ -151,8 +150,7 @@ class Sampler():
             result of the multiplication.
         """
         if isinstance(other, (float, int)):
-            self.weight *= other
-            return self
+            return WeightedSampler(self, self.weight * other)
 
         return AndSampler(self, other)
 
@@ -235,8 +233,13 @@ class OrSampler(Sampler):
 
         up_sample = self.bases[0].sample(size=up_size)
         low_sample  = self.bases[1].sample(size=low_size)
-        sample_points = np.concatenate([up_sample, low_sample])
-        sample_points = sample_points[np.random.permutation(size)]
+        if len(up_sample) > 0 and len(low_sample) > 0:
+            sample_points = np.concatenate([up_sample, low_sample])
+            sample_points = sample_points[np.random.permutation(size)]
+        elif len(up_sample) == 0:
+            sample_points = low_sample
+        else:
+            sample_points = up_sample
 
         return sample_points
 
@@ -341,6 +344,20 @@ class TruncateSampler(Sampler):
 
         return np.concatenate(samples)[:size]
 
+class WeightedSampler(Sampler):
+    """ Class for implementing `&` (weighting) operation on a number and a `Sampler` instance.
+    """
+    def __init__(self, base, weight, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.bases = [base]
+        self.weight = weight
+
+    def sample(self, size):
+        """ Sampling procedure of a product of the number and the sampler instance. Check out the docstring of
+        `Sampler.sample` for more info.
+        """
+        return self.bases[0].sample(size)
+
 
 class BaseOperationSampler(Sampler):
     """ Base class for implementing all arithmetic operations on `Sampler`-instances.
