@@ -1,11 +1,9 @@
 """ Tests for FilesIndex class. """
-# pylint: disable=missing-docstring
-# pylint: disable=protected-access
-# pylint: disable=redefined-outer-name
+
 import os
 import shutil
 
-from contextlib import ExitStack as does_not_raise
+from contextlib import ExitStack as does_not_raise  # noqa: N813; camelcase-imported-as-lowercase
 
 import pytest
 import numpy as np
@@ -25,7 +23,8 @@ def files_setup(request):
     for folder in folders:
         os.mkdir(folder)
         for i in range(3):
-            open(os.path.join(folder, f'file_{i}.txt'), 'w').close()
+            with open(os.path.join(folder, f'file_{i}.txt'), 'w', encoding='utf-8'):
+                pass
 
     def fin():
         shutil.rmtree(path)
